/**
 * Author: Claudia D'Ambrosio
 * Date: 2012/02/14
 * XTMINLP.cpp
 */

#include "MyTMINLP.hpp"
#include "BonAmplInterface.hpp"

bool 
MyTMINLP::get_variables_types(Index n, VariableType* var_types)
{
  var_types[0] = CONTINUOUS;
  var_types[1] = INTEGER;
  return true;
}


bool 
MyTMINLP::get_variables_linearity(Index n, Ipopt::TNLP::LinearityType* var_types)
{
  var_types[0] = Ipopt::TNLP::NON_LINEAR;
  var_types[1] = Ipopt::TNLP::NON_LINEAR;
  return true;
}


bool 
MyTMINLP::get_constraints_linearity(Index m, Ipopt::TNLP::LinearityType* const_types)
{
  assert (m==0);
  return true;
}
bool 
MyTMINLP::get_nlp_info(Index& n, Index&m, Index& nnz_jac_g,
                       Index& nnz_h_lag, TNLP::IndexStyleEnum& index_style)
{
  n = 4;//number of variable
  m = 0;//number of constraints
  nnz_jac_g = 0;//number of non zeroes in Jacobian
  nnz_h_lag = 4;//number of non zeroes in Hessian of Lagrangean
  index_style = TNLP::FORTRAN_STYLE;
  return true;
}

bool 
MyTMINLP::get_bounds_info(Index n, Number* x_l, Number* x_u,
                            Index m, Number* g_l, Number* g_u)
{
  assert(n==2);
  assert(m==0);
  x_l[0] = -2.;
  x_u[0] = 2.;
  
  x_l[1] = -1.;
  x_u[1] = 1.;
  
  return true;
}

bool 
MyTMINLP::get_starting_point(Index n, bool init_x, Number* x,
                             bool init_z, Number* z_L, Number* z_U,
                             Index m, bool init_lambda,
                             Number* lambda)
{
  assert(n==4);
  assert(m==0);
  
  assert(init_x);
  assert(!init_lambda);
  x[0] = 2.;
  x[1] = -1.;
  return true;
}

bool 
MyTMINLP::eval_f(Index n, const Number* x, bool new_x, Number& obj_value)
{
  assert(n==2);
  obj_value = 4.*pow(x[0],2.) - 2.1*pow(x[0],4.) + (0.3333333)*pow(x[0],6.) + x[0]*x[1] - 4.*pow(x[1],2.) + 4.*pow(x[1],4.);
  return true;
}

bool
MyTMINLP::eval_grad_f(Index n, const Number* x, bool new_x, Number* grad_f)
{
  assert(n==2);
  grad_f[0] = 8.*x[0]-8.4*pow(x[0],3.)+2.*pow(x[0],5.)+x[1];
  grad_f[1] = x[0]-8.*x[1]+16.*pow(x[1],3.);
  return true;
}

bool
MyTMINLP::eval_g(Index n, const Number* x, bool new_x, Index m, Number* g)
{
  assert(n==2);
  assert(m==0);
  
  return true;
}

bool
MyTMINLP::eval_jac_g(Index n, const Number* x, bool new_x,
                     Index m, Index nnz_jac, Index* iRow, Index *jCol,
                     Number* values)
{
  assert(n==4);
  assert(nnz_jac == 0);
  return true;
}

bool
MyTMINLP::eval_h(Index n, const Number* x, bool new_x,
                 Number obj_factor, Index m, const Number* lambda,
                 bool new_lambda, Index nele_hess, Index* iRow,
                 Index* jCol, Number* values)
{
  assert (n==2);
  assert (m==0);
  assert(nele_hess==4);
  if(values==NULL)
  {
    iRow[0] = 1;
    jCol[0] = 1;
    
    iRow[1] = 1;
    jCol[1] = 2;

    iRow[2] = 2;
    jCol[2] = 1;

    iRow[3] = 2;
    jCol[3] = 2;
  }
  else {
    values[0] = obj_factor*(8.-25.2*pow(x[0],2.)+10.*pow(x[0],4.));
    values[1] = obj_factor*(1);
    values[2] = obj_factor*(1);
    values[3] = obj_factor*(-8.+48.*pow(x[1],2.));
  }
  return true;
}

void
MyTMINLP::finalize_solution(TMINLP::SolverReturn status,
                            Index n, const Number* x, Number obj_value)
{
  std::cout<<"Problem status: "<<status<<std::endl;
  std::cout<<"Objective value: "<<obj_value<<std::endl;
  if(printSol_ && x != NULL){
    std::cout<<"Solution:"<<std::endl;
      std::cout<<"x = "<<x[0];
      std::cout<<", ";}
      std::cout<<"y = "<<x[1];
    std::cout<<std::endl;

}
